// peertube-instance-index-filter
// Copyright (C) 2025 Arns Udovič <zordsdavini@arns.lt>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package main

import (
	"encoding/json"
	"fmt"
	"net/http"
	"strings"
	"time"
)

func doAudit(host string, filter string) bool {
	db := connectDB()
	defer db.Close()

	switch filter {
		case "dead":
			_, err := http.Get("https://" + host)
			if err != nil {
				return true
			}
		case "empty":
			resp, err := http.Get("https://" + host + "/api/v1/server/stats")
			if err != nil {
				panic(err)
			}

			defer resp.Body.Close()	

			if resp.StatusCode != http.StatusOK {
				return false
			}

			var generic map[string]any
		    err = json.NewDecoder(resp.Body).Decode(&generic)
		    if err != nil {
				panic(err)
		    }

			videos := generic["totalLocalVideos"].(float64)
			if videos != 0 {
				return false
			}

			instance, err := getInstance(db, host)
			if err != nil {
				panic(err)
			}

			currentTime := time.Now()
			last3Month := currentTime.AddDate(0,-3,0)
			if strings.Compare(instance.CreatedAt, last3Month.Format("2006-01-02T15:04:05Z")) < 1 {
				return true
			}
	}

	return false
}

func removeDuplicates() {
	db := connectDB()
	defer db.Close()

	for {
		ids, err := getDuplicateIds(db)
		if err != nil {
			panic(err)
		}

		if len(ids) == 0 {
			break
		}

		fmt.Println("Removing duplicates:", len(ids), ids)

		for _, id := range ids {
			removeInstance(db, id)
		}
	}
}
